"""
Models for ResumeManager app.
"""

# Third-Party Imports.
from django.db import models
from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils import timezone

# Internal Imports.
from core.models import WorkspaceModel


class Resume(WorkspaceModel):
    """A representation of a resume."""

    # Relationship Keys.
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    theme = models.ForeignKey('ResumeTheme', on_delete=models.CASCADE)

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    last_activity = models.DateField(default=timezone.now)

    # Field helper text.
    name.help_text = 'Resume name.'
    last_activity.help_text = 'Last set date of resume updating. Defaults to creation date. Manually set after.'

    class Meta:
        verbose_name = 'Resume'
        verbose_name_plural = 'Resumes'

    def __str__(self):
        return '{0} - {1}'.format(self.name, self.last_activity)


class ResumeTheme(WorkspaceModel):
    """A unique resume theme."""

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    css_classes = models.CharField(max_length=settings.MAX_LENGTH)
    has_header_image = models.BooleanField(default=False)
    has_footer_image = models.BooleanField(default=False)

    # Field helper text.
    name.help_text = 'Name of theme.'
    has_header_image.help_text = 'Indicates if theme includes header image.'
    has_footer_image.help_text = 'Indicates if theme includes footer image.'

    class Meta:
        verbose_name = 'Resume Theme'
        verbose_name_plural = 'Resume Themes'

    def __str__(self):
        return '{0}'.format(self.name)


class ResumeSection(WorkspaceModel):
    """Formatting of a section/subsection within a resume."""
    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    css_classes = models.CharField(max_length=settings.MAX_LENGTH)

    # Field helper text.
    name.help_text = 'Name of resume section.'
    css_classes.help_text = 'CSS classes to apply for section.'

    class Meta:
        verbose_name = 'Resume Section'
        verbose_name_plural = 'Resume Sections'

    def __str__(self):
        return '{0}'.format(self.name)


class ResumeToSectionIntermediary(WorkspaceModel):
    """An intermediary model that associates a resume and a section.

    Can be nested within other section/subsections, denoted by having a "parent".
    Each subsection is a unique div.
    """

    # # Preset field choices.
    # DISPLAY_CATEGORY__NONE = 0
    # DISPLAY_CATEGORY__EXPERIENCE = 1
    # DISPLAY_CATEGORY__EDUCATION = 2
    # DISPLAY_CATEGORY__SKILLS = 3
    # DISPLAY_CATEGORY_CHOICES = (
    #     (DISPLAY_CATEGORY__NONE, 'None'),
    #     (DISPLAY_CATEGORY__EXPERIENCE, 'Experience'),
    #     (DISPLAY_CATEGORY__EDUCATION, 'Education'),
    #     (DISPLAY_CATEGORY__SKILLS, 'Skills'),
    # )

    # Relationship Keys.
    resume = models.ForeignKey('Resume', on_delete=models.CASCADE)
    parent_section = models.ForeignKey('self', on_delete=models.CASCADE, blank=True, null=True)
    section_format = models.ForeignKey('ResumeSection', on_delete=models.CASCADE)
    is_data_subsection = models.BooleanField(default=False)
    # display_category = models.PositiveSmallIntegerField(choices=DISPLAY_CATEGORY_CHOICES, default=0)

    # Field helper text.
    resume.help_text = 'The resume to place the section within.'
    parent_section.help_text = 'Parent section/subsection to nest within.'
    section_format.help_text = 'The section formatting to use within the resume.'
    is_data_subsection.help_text = 'Indicates if this is part of a larger parent data section. Results in de-emphasized text.'
    # display_category.help_text = 'Category of data to display in section.'

    class Meta:
        verbose_name = 'Resume-to-Section Intermediary'
        verbose_name_plural = 'Resume-to-Section Intermediaries'

    def __str__(self):
        return '{0} - {1}'.format(self.resume, self.section_format)


# region Resume Data

class DataSectionHeader(WorkspaceModel):
    """Implementation of a header for a data section.

    Each data section can only have a single header, but one is not required.
    """

    # Relationship Keys.
    resume_section = models.ForeignKey('ResumeToSectionIntermediary', on_delete=models.CASCADE)

    # Model fields.
    section_name = models.CharField(max_length=settings.MAX_LENGTH)
    header = models.TextField(blank=True)

    class Meta:
        verbose_name = 'Data Section Header'
        verbose_name_plural = 'Data Section Headers'

    def __str__(self):
        return '{0} - {1}'.format(self.resume_section.resume, self.section_name)


class DataItem(WorkspaceModel):
    """Abstract implementation of a data item.

    If a data section has only a single item, then the item is displayed in block/paragraph format.
    If a data section has two or more items, then they are displayed in bullet point format.
    """

    # Relationship Keys.
    resume_section = models.ForeignKey('ResumeToSectionIntermediary', on_delete=models.CASCADE)

    # Model fields.
    section_name = models.CharField(max_length=settings.MAX_LENGTH)
    item = models.TextField(blank=True)
    is_emphasized = models.BooleanField(default=False)
    is_minimized = models.BooleanField(default=False)

    class Meta:
        verbose_name = 'Data Section Item'
        verbose_name_plural = 'Data Section Items'

    def __str__(self):
        return '{0} - {1}'.format(self.resume_section.resume, self.section_name)

    def clean(self):
        """
        Custom cleaning implementation. Includes validation, setting fields, etc.
        """
        # Call parent logic.
        super().clean()

        if self.is_emphasized and self.is_minimized:
            raise ValidationError('Data cannot be both emphasized and minimized in style.')

# endregion Resume Data
